"""
Class for managing multiple time trials in our protein code. writePath
is the directory where data files will be written to. When all timing is done,
writes three output files to disk - timekeeper-records.log, timekeeper-error.log,
and elapsedTimeRecords.p, a pickle file containing the elapsed timing data for
each time trial.

Brief tutorial:

import timekeeper

myTimer = timekeeper.timekeeper('/home/myStuff/timekeeperOutput')

#Now that we have our timer, we can start making time trials.

myTimer.startTimeTrial('test1')
... do stuff ...
myTimer.startTimeTrial('test2')
... do stuff ...
myTimer.stopTimeTrial('test1')
myTimer.stopTimeTrial('test2')

#Then conclude all timing.
myTimer.concludeTiming()

...do whatever...

#Now write the output to disk.
myTimer.writeTimeOutput()

Brad Ridder, April 19, 2012, Purdue University, Department of Chemical
Engineering, Chakrabarti Group
"""
import time
import datetime
import checkSlash
import pickle

class timekeeper:
	def __init__(self,writePath):
		
		#Start global timing.
		self.globalStartTime = time.time()
		startDatetime = str(datetime.datetime.now())
		
		#Check the writePath and save it.
		writePath = checkSlash.checkSlash(writePath)
		self.writePath = writePath

		#Initialize warning count.
		self.warningCount = 0
		
		#Initialize time records as empty dictionaries.
		self.startTimeRecord = dict()
		self.elapsedTimeRecord = dict()

		#Open timeData and errorData files. timeDataFile holds the elapsedTimeRecords,
		#and errorDataFile holds warning messages.
		
		self.timeDataFile = open(self.writePath + 'timekeeper-records.log','w')
		self.errorDataFile = open(self.writePath + 'timekeeper-error.log','w')
		
		#Put date and time into the timeDataFile.
		self.timeDataFile.write('Timing started on %s' %(startDatetime) + '\n')
		
	def startTimeTrial(self,trialName):
		'''
		Starts a time trial, designated by the string trialName.
		'''
		self.startTimeRecord[trialName] = time.time()
		
	def stopTimeTrial(self,trialName):
		'''
		Stops timing for the time trial with trialName. Records the elapsed time,
		and writes a warning to timekeeper-error.log if the trial was stopped, but
		never started.
		'''
		try:
			self.elapsedTimeRecord[trialName] = time.time() - self.startTimeRecord[trialName]
		#This exception accounts for when someone tries to stop a time trial that
		#was never started.
		except KeyError:
			self.warningCount += 1
			warningTime = time.time() - self.globalStartTime
			self.errorDataFile.write('Warning %s: Attempted to stop time trial \'%s\', but it was never started.\n' %(self.warningCount,trialName))
			self.errorDataFile.write('Warning %s: Seconds into run when attempted to stop time trial: %s.\n' %(self.warningCount,warningTime))
			
	def concludeTiming(self):
		'''
		This method forces all timing tests to stop, and prints error messages to timekeeper-error.log for
		trials that were started, but never stopped.
		'''
		#Check to make sure all tests that were started have stopped. If there is time trial
		#that was started, but never stopped, it will be stopped now, and an error written to the error log.
		setOfKeysA = set(self.startTimeRecord.keys())
		setOfKeysB = set(self.elapsedTimeRecord.keys())
		#If both sets are equal, their difference will be a null set. "Not null"
		#is "True" in Python.
		diffSet = (setOfKeysA - setOfKeysB)
		if not diffSet:
			pass
		#Else, we write the neglected trials to the error log.
		else:
			while diffSet:
				self.warningCount += 1
				trialName = diffSet.pop()
				warningTime = self.startTimeRecord[trialName] - self.globalStartTime
				self.errorDataFile.write('Warning %s: Forced end of time trial \'%s\'; it was started, but never stopped.\n' %(self.warningCount,trialName))
				self.errorDataFile.write('Warning %s: Seconds into run when time trial was started: %s.\n' %(self.warningCount,warningTime))
			self.errorDataFile.close()
			
	def writeTimeOutput(self):
		'''
		Writes all timing data to a text file 'timekeeper-records.log', and also
		writes the elapsedTimeRecords dictionary to a pickle file.
		'''
		endTime = str(datetime.datetime.now())
		for key in self.elapsedTimeRecord:
			timeAsString = str(self.elapsedTimeRecord[key])
			self.timeDataFile.write(key + ' ' + timeAsString + '\n')
		stopDatetime = str(datetime.datetime.now())
		self.timeDataFile.write('Timing concluded on %s.' %(stopDatetime) + '\n')
		self.timeDataFile.write('Total elapsed time: %s.\n' %(time.time() - self.globalStartTime))
		self.timeDataFile.write('Total number of warnings: %s.' %(self.warningCount))
		self.timeDataFile.close()
		pickle.dump(self.elapsedTimeRecord,open('elapsedTimeRecord.p', 'wb'))
